#!/usr/bin/bash
set -e

echo > /tmp/sport.log

# associative array to track used ports
declare -A used
# pre-block the privileged ports
for n in $(seq 1 1023); do
  used[$n]=1
done

# IPv4 defaults
ipv4_ip=${ipv4_ip:-203.0.113.0}

# IPv6 defaults
ipv6_ip=${ipv6_ip:-fd00::1}

# session defaults
max=${max:-10}
port=${port:-1234}
message=${message:-Hello}
wt=${wt:-0}
protocol=${protocol:--u}
 
if [ -f /tmp/ecmp.conf ]; then
  read ip_family max < /tmp/ecmp.conf
fi

if [ -n "$1" ]; then
  ip_family="$1"
fi
ip_family=${ip_family:--4}

# operator inet family choice for session
case "$ip_family" in
  "-4")
    ip_anycast=$ipv4_ip
    ;;

  "-6")
    ip_anycast=$ipv6_ip
    ;;
esac
count=${count:-10000}

echo "Starting $count tests with $max namespaces using protocol $protocol address family $ip_family @ port $port" 

for nsnum in $(seq 1 $max); do
  ip netns exec ns$nsnum nc $ip_family -k $protocol -l $port  > /tmp/ns$nsnum.log &
done
jobs

while [ $count -gt 0 ]; do
  count=$((count -1))
  while /bin/true; do
    sport=$(($RANDOM+$RANDOM))
    if [ -z ${used[$sport]} ]; then
      used[$sport]=1
      break
    fi
  done

  echo $sport >> /tmp/sport.log
  echo "$message $count $sport " | nc $ip_family $protocol -w $wt -p $sport $ip_anycast $port || echo "error: $count $sport"
  if [ $((count %1000)) -eq 0 ]; then
    echo $count
  fi
done

for nsnum in $(seq 1 $max); do
  ip netns pids ns$nsnum | while read pid; do
    kill $pid
  done
done

for f in /tmp/ns*.log; do 
  wc -l $f
  rm $f
done | sort -n
	
