#!/usr/bin/bash
set -e

# Layer 3+4 5-tuple hash
echo 1 > /proc/sys/net/ipv4/fib_multipath_hash_policy
echo 1 > /proc/sys/net/ipv6/fib_multipath_hash_policy
# learn quickly of dead hosts so they are removed from the hash table
echo 1 > /proc/sys/net/ipv4/fib_multipath_use_neigh

# IPv4 defaults
ipv4_prefix=${ipv4_prefix:-198.51.100.0/24}
ipv4_default=${ipv4_default:-0.0.0.0/0}
ipv4_host=${ipv4_host:-198.51.100.0/32}
ipv4_anycast=${ipv4_anycast:-203.0.113.0/32}

# IPv6 defaults
ipv6_prefix=${ipv6_prefix:-fd00:b00b:1e00::/64}
ipv6_default=${ipv6_default:-::/0}
ipv6_host=${ipv6_host:-100::1/128}
ipv6_anycast=${ipv6_anycast:-fd00::1/128}

# session defaults
ip_family=${ip_family:--4}
ip_default=${ipv4_default}
ip_host=${ipv4_host}
ip_anycast=${ipv4_anycast}

# operator inet family choice for session
case "$1" in 
  "-4")
    ip_family="$1"
    ip_default=$ipv4_default
    ip_host=$ipv4_host
    ip_anycast=$ipv4_anycast
    ;;

  "-6")
    ip_family="$1"
    ip_default=$ipv6_default
    ip_host=$ipv6_host
    ip_anycast=$ipv6_anycast
    ;;
esac

# number of hosts participating in load-balancing the anycast address
max=${max:-10}

echo "$ip_family $max" > /tmp/ecmp.conf

# routes back to host targets this address
ip addr add $ip_host dev lo

# range = 1-255
nsnum=1
# base interface names INside and OUTside namespaces
vethin=2000
vethout=1000
# base of the anycast route command required to create multi-host nexthop
cmd="ip route add $ip_anycast"
# create and configure the network namespaces
for nsnum in $(seq 1 $max); do
        vethin="veth$((2000 + nsnum))"
        vethout="veth$((1000 + nsnum))"
        echo "nsnum=$nsnum vethin=$vethin vethout=$vethout"

	ip netns add ns${nsnum}
        ip link add $vethout type veth peer name $vethin
        ip link set $vethout up
	# move peer into namespace
        ip link set $vethin netns ns${nsnum}
        ip -n ns${nsnum} link set $vethin name eth0
        ip -n ns${nsnum} link set eth0 up

	# IP address inside network namespace
	if [ "x$ip_family" = "x-4" ]; then
	  if_ip="192.0.2.${nsnum}/32"
	  via=""
        else
          # IPv6 needs routable address on OUTside and prefix to allow it
	  if_ip="$(printf "${ipv6_prefix%/*}%x:2/112" $nsnum)"
          ip addr add ${if_ip/:2\//:1\/} dev $vethout
	  ip -n ns${nsnum} route add ${if_ip/:2\//:1\/} dev eth0
	  # IPv6 needs default route via on-link address of outside peer
	  via="via ${if_ip/:2\//:1\/}"
	fi
	# one more load-balancer for the anycast route
        cmd="$cmd nexthop via ${if_ip%/*} weight 10"
        ip -n ns${nsnum} addr add $if_ip dev eth0
	# route back to host
        ip -n ns${nsnum} route add $ip_host dev eth0
	# default route via host (via only added for IPv6)
        ip -n ns${nsnum} route add $ip_default ${via%/*} dev eth0
        ip route add $if_ip dev $vethout
	# attach anycast address to the namespace
        ip -n ns${nsnum} addr add $ip_anycast dev lo
        ip -n ns${nsnum} link set dev lo up 
done
# add ECMP anycast route
$cmd
